/***************************************************************************
 *   Copyright (C) 2005 - 2006 by Alexander Saal                           *
 *   alex.saal@gmx.de                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "vsprogen.h"

#include <QtGui>
#include <QtCore>

VSProGen::VSProGen(QWidget *parent) : QWidget(parent)
{
	// init the UI class, without ui. .... *brrrr* is not nice if you use ui.setupUi(this)
	// i like setupUi(this) without ui. please look at vsprogen.h for more informtion :)
	// i mean this: class ..., ..., public Ui::UiVSProGenBase { //... }
	setupUi(this);

	// connection (i not like automated connection .... *brrrrr-bhh-iiiiiiiiiigiiiiit*) :)
	connect( btnOpenQtProFile, SIGNAL( clicked() ), this, SLOT( slotOpenQtProjectFile() ) );
	connect( btnCreate, SIGNAL( clicked() ), this, SLOT( slotCreate() ) );

	// center the application on desktop screen - i love it ;)
	QDesktopWidget *desktop = qApp->desktop();
    const QRect rect = desktop->availableGeometry( desktop->primaryScreen() );
    int left = ( rect.width() - width() ) / 2;
    int top = ( rect.height() - height() ) / 2;
    setGeometry( left, top, width(), height() );
}

VSProGen::~VSProGen()
{}

/* Open Qt4 Projectfile (*.pro) */
void VSProGen::slotOpenQtProjectFile()
{
	/* Declare a QString variabel, open a QFileDialog and select a file and write this to the variabel */
	fileName = QFileDialog::getOpenFileName( this, "Choose Qt4 Projectfile", QCoreApplication::applicationDirPath(), "Qt4 Projectfile (*.pro)");

	/* check if the variabel not empty and fill the QLineEdit with this selected file */
	if( !fileName.isEmpty() ) {
		btnCreate->setEnabled( true );
		lineEditQtProFile->setText(fileName);
	} else {
		btnCreate->setEnabled( false );
	}

}

/* Create Visual C++ ... Projectfile */
void VSProGen::slotCreate()
{
	// we need a little bit more information about this file ;)
	QFileInfo vsProjectFilePath( fileName );

	// create a new instance of QProcess
	QProcess *procCreateVs = new QProcess( this );

	// set the working directory to the file path of Qt Projectfile, you see, we needed the QFileInfo Object ;)
	procCreateVs->setWorkingDirectory( vsProjectFilePath.absolutePath() + QDir::separator() );

	// oh it is now complicated ;)
	// but easy to use .... i switch with the selected combobox item
	// and than i file the QStringList with life
	switch ( cbxVSOption->currentIndex() ) {
        case 0:               // Visual C++ 6.0 (*.dsp)
			proFileVs.append( "-spec" );
			proFileVs.append( "win32-msvc" );
			proFileVs.append( "-t" );
			proFileVs.append( "vcapp" );
			proFileVs.append( "-o" );
			proFileVs.append( vsProjectFilePath.baseName() + ".dsp" ); // Outputfile (*.dsp)
			proFileVs.append( lineEditQtProFile->text() ); // Qt Projectfile (*.pro)
            break;
        case 1:               // Visual C++ .Net 2003 (*.vcproj)
			proFileVs.append( "-spec" );
			proFileVs.append( "win32-msvc.net" );
			proFileVs.append( "-t" );
			proFileVs.append( "vcapp" );
			proFileVs.append( "-o" );
			proFileVs.append( vsProjectFilePath.baseName() + ".vcproj" ); // Outputfile (*.vcproj)
			proFileVs.append( lineEditQtProFile->text() ); // Qt Projectfile (*.pro)
            break;
        case 2:               // Visual C++ .Net 2005 (*.vcproj)
			proFileVs.append( "-spec" );
			proFileVs.append( "win32-msvc2005" );
			proFileVs.append( "-t" );
			proFileVs.append( "vcapp" );
			proFileVs.append( "-o" );
			proFileVs.append( vsProjectFilePath.baseName() + ".vcproj" ); // Outputfile (*.vcproj)
			proFileVs.append( lineEditQtProFile->text() ); // Qt Projectfile (*.pro)
            break;        
    }
	
	// ok, all requirements are available, now i start the QProcess
	// and connect this to another slots. i need more information for this ;)
	procCreateVs->start( "qmake", proFileVs );
    connect( procCreateVs, SIGNAL( finished( int, QProcess::ExitStatus ) ), this, SLOT( finish( int, QProcess::ExitStatus ) ) );
    connect( procCreateVs, SIGNAL( error( QProcess::ProcessError ) ), this, SLOT( err( QProcess::ProcessError ) ) );

	// all finish, clear the QStringList
	// is needed, if not clear..., it comes to problems
	// with next start of this QProcess...
	// clear ... it is better so, ok? ;)
	proFileVs.clear();
}

/* QProcess was finish ... */
void VSProGen::finish( int exitCode, QProcess::ExitStatus exitStatus )
{
    switch ( exitStatus ) {
        case QProcess::NormalExit:
            lblInfo->setText( "qmake: process exited normally" );
            break;
        case QProcess::CrashExit:
            lblInfo->setText( "qmake: process crashed" );
            break;
    }

	switch( QMessageBox::information( this, "(Qt4) VSGen", "Would you like to replace the absolut Qt4 path\n"
											"with relative Qt4 path '$(QTDIR)' in the created\n"
											"project file?", "Yes", "No", "Cancel", 0, 2 ) ) {
		case 0:               // Yes clicked
			replaceQtAbsolutPath();
			break;
		case 1:               // No clicked
			break;
		case 2:               // Cancel clicked or Escape pressed
			break;
	}
}

// qmake process was exited and have an error returned, but what for a error? ;)
void VSProGen::err( QProcess::ProcessError error )
{
    switch ( error ) {
        case QProcess::FailedToStart:
            lblInfo->setText( "qmake: the dosbox qmake failed to start" );
            break;
        case QProcess::Crashed:
            lblInfo->setText( "qmake: dosbox process crashed some time after starting successfully" );
	        break;
        case QProcess::Timedout:
            lblInfo->setText( "qmake: last waitFor...() function timed out" );
            break;
        case QProcess::WriteError:
            lblInfo->setText( "qmake: an error occurred when attempting to write to the qmake process" );
            break;
        case QProcess::ReadError:
            lblInfo->setText( "qmake: an error occurred when attempting to read from the qmake process" );
            break;
        case QProcess::UnknownError:
            lblInfo->setText( "qmake: An unknown error occurred" );
            break;
    }
}

void VSProGen::replaceQtAbsolutPath()
{
	QFileInfo vsProjectFilePath( fileName );
	QString tmpLine, qtDir, qtDirOriginal, strTmp, suffix, patchFile, patchedFile;
	QStringList tmp;

	switch ( cbxVSOption->currentIndex() ) {
        case 0:               // Visual C++ 6.0 (*.dsp)
			patchFile = vsProjectFilePath.absolutePath() + QDir::separator() + vsProjectFilePath.baseName() +  ".dsp";
			suffix = ".dsp";
            break;
        case 1:               // Visual C++ .Net 2003 (*.vcproj)
			patchFile = vsProjectFilePath.absolutePath() + QDir::separator() + vsProjectFilePath.baseName() +  ".vcproj";
			suffix = ".vcproj";
			break;
        case 2:               // Visual C++ .Net 2005 (*.vcproj)
			patchFile = vsProjectFilePath.absolutePath() + QDir::separator() + vsProjectFilePath.baseName() +  ".vcproj";
			suffix = ".vcproj";
			break;  
    }

	QFile tmpFile( patchFile );
	if( !tmpFile.open(  QFile::ReadWrite | QFile::Text ) ) {
		lblInfo->setText( "File: " + vsProjectFilePath.baseName() + suffix + " not patched - <b>ERROR</b>, file exists?" );
		return;
	}

	// Get QtDir from Systemenvironment or Userenvironment (it is Qt4 Based)
	QByteArray defaultQtDir = qgetenv("QTDIR");

	// convert to readable ;)
	qtDir = QString::fromLocal8Bit( defaultQtDir );

	// add a windows seperator ... like this: '\' and letter this: '/'
	// look down for more information :)
	qtDir.append( "\\" );

	// create QTextStream and read all data into this stream
	// then close file and at finaly, remove it ;)
	QTextStream t( &tmpFile );
	tmpLine = t.readAll();
	tmpFile.close();
	tmpFile.remove();

	// first step: beginn with replace
	tmpLine.replace( qtDir, "$(QTDIR)\\" );

	tmp = qtDir.split(":");
	strTmp = tmp.value(0).toLower() + ":" + tmp.value(1);
	tmpLine.replace( strTmp, "$(QTDIR)\\" );

	// second step: begin with replace
	qtDir.replace( "\\", "/" );
	tmpLine.replace( qtDir, "$(QTDIR)/" );

	tmp.clear();
	strTmp = "";
	tmp = qtDir.split(":");
	strTmp = tmp.value(0).toLower() + ":" + tmp.value(1);
	tmpLine.replace( strTmp, "$(QTDIR)/" );


	// at finaly, we write the result to new file
	// and rename this file with correct suffix
	patchedFile = vsProjectFilePath.absolutePath() + QDir::separator() + vsProjectFilePath.baseName() + ".vsprogen";
	QFile tmpFile1( patchedFile );
	if( !tmpFile1.open(  QFile::WriteOnly | QFile::Text ) ) {
		lblInfo->setText( "File: " + vsProjectFilePath.baseName() + suffix + " not patched - <b>ERROR</b>, file exists?" );
		return;
	}

	QTextStream t1( &tmpFile1 );
	t1 << tmpLine;
	t1.flush();
	tmpFile1.close();
	tmpFile1.rename( vsProjectFilePath.absolutePath() + QDir::separator() + vsProjectFilePath.baseName() + suffix );
	lblInfo->setText( "File: " + vsProjectFilePath.baseName() + suffix + " <b>successful</b> patched" );
}
